"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const axios_1 = __importDefault(require("axios"));
const randomstring_1 = __importDefault(require("randomstring"));
const crypto_js_1 = __importDefault(require("crypto-js"));
const sortObjectKeys_1 = require("./helpers/sortObjectKeys");
const contest_1 = require("./modules/contest");
class CodeforcesClient {
    constructor(key = '', secret = '') {
        this.key = key;
        this.secret = secret;
        this.contest = contest_1.contest(this);
        this._baseUrl = 'http://codeforces.com/api';
    }
    callMethod(method, params) {
        return __awaiter(this, void 0, void 0, function* () {
            const url = this.generateUrl(method, params);
            try {
                const { data } = yield axios_1.default.get(url);
                return data;
            }
            catch (error) {
                return error.response.data;
            }
        });
    }
    generateUrl(method, params) {
        const randomStart = randomstring_1.default.generate(6);
        const time = Math.round(Date.now() / 1000);
        const apiSigParamsObject = sortObjectKeys_1.sortObjectKeys(Object.assign(Object.assign({}, params), { apiKey: this.key, time }));
        const apiSig = this.generateApiSig(randomStart, method, apiSigParamsObject);
        return `${this._baseUrl}/${method}?${this.toParamsString(params)}&apiKey=${this.key}&time=${time}&apiSig=${randomStart}${apiSig}`;
    }
    generateApiSig(randomStart, methodName, params) {
        return crypto_js_1.default.SHA512(`${randomStart}/${methodName}?${this.toParamsString(params)}#${this.secret}`).toString();
    }
    toParamsString(params) {
        return Object.keys(params)
            .map(key => `${key}=${params[key]}`)
            .join('&');
    }
}
exports.CodeforcesClient = CodeforcesClient;
//# sourceMappingURL=CodeforcesClient.js.map